function [ imageCoordinates ] = imageProjection( X, P)
%% This function projects to 3D points to the image plane given a projection matrix
%INPUT:
%  X: 3xN coordinates of N points, or 4xN homogeneous coordinates
%  P: projection matrix 
%OUTPUT:
% 2xN image coordinates
%
% This function is part of the MPI08 data set, to be obtained at 
% http://www.tnt.uni-hannover.de/project/MPI08_Database/
%
% Written by: Andreas Baak
% Revision:   1.0
% Date:       01.10.2010
%
% Copyright 2010, MPI Informatik, Germany, and Hannover University, Germany.
% All Rights Reserved Permission to use, copy, modify, and distribute this
% software and its documentation for any non-commercial purpose is hereby
% granted without fee, provided that the above copyright notice appear in
% all copies and that both that copyright notice and this permission
% notice appear in supporting documentation, and that the name of the
% author not be used in advertising or publicity pertaining to distribution
% of the software without specific, written prior permission.
% THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
% INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR ANY
% PARTICULAR PURPOSE. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
% SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
% RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
% CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
% CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

numPoints = size(X, 2);
if size(X,1) == 3
    X(4,:) = ones(1, numPoints);
elseif size(X,1) ~= 4
    error('X has to be either 3xN coordinates or 4xN homogeneous coordinates.');
end
PX = P * X;
imageCoordinates = [PX(1,:)./PX(3,:); PX(2,:)./PX(3,:)];
end